/**
 * Created by jmoutard on 17/05/16.
 */

'use strict';

angular.module('adf.electre')
    .controller('navigationCtrl', [
        "$scope", "$rootScope", "$q", "$location", "storeService", "defaultDashboard", "$timeout",
        function ($scope, $rootScope, $q, $location, storeService, defaultDashboard, $timeout) {
            /**
             * @description Display errors from the storage service.
             * @param {object} error Error from the storage service.
             */
            function displayError (error) {
                $scope.errorMessage = (error.message ? error.message() : error);
                $timeout(function () {
                    $scope.errorMessage = false;
                }, 2500);
            }
            /**
             * @description Taking the dashboards from the storage. Creating objects for the navbar menu.
             * @param {object} values Data given by the storage.
             */
            function takeDashboards (values) {
                /**
                 * @description Get the items names and create a link to access each dashboard
                 * @param {Array} items
                 * @param {boolean|undefined} remote Tells if corresponds to local or remote items
                 * @returns {Array}
                 */
                function getItem (items, remote) {
                    var response = [];
                    for (var _i = 0, item = items[_i] ;
                         _i < items.length;
                         _i++, item = items[_i]
                    ) {
                        response.push({"name" : item.title, "link": (remote ? "remote_board" : "local_board") + "({id: \"" + item.id + "\"})"});
                    }
                    return response;
                }

                if (values && typeof values === "object") {
                    if (values.serverStorage && values.serverStorage.length > 0) {
                        nav.serverDashboard = true;
                        $scope.serverDashboardTree = getItem(values.serverStorage, true);
                    }
                    if (values.localStorage && values.localStorage.length > 0) {
                        nav.localDashboard = true;
                        $scope.localDashboardTree = getItem(values.localStorage, false);
                        console.log(nav, $scope);
                    }
                }
            }
            /**
             * @description Removing all dashboard's object in order to clear the navbar menu.
             */
            function resetDashboards () {
                nav.localDashboard = false;
                nav.serverDashboard = false;
                $scope.serverDashboardTree = undefined;
                $scope.localItem = undefined;
            }
            
            var nav = this;
            nav.navCollapsed = false;

            $scope.tree = [
                {
                    name: "Home",
                    link: "home"
                },{
                    name: "Log in",
                    link: "login"
                },{
                    name: "Log out",
                    link: "logout"
                }
            ];
            $scope.serverDashboardTree = [
                {
                    name: "title",
                    link: "remote_board"
                }
            ];
            $scope.localDashboardTree = [
                {
                    name:"title",
                    link:"local_board"
                }
            ];

            $scope.toggleNav = function(){ nav.navCollapsed = !nav.navCollapsed; };

            $scope.navClass = function(page) {
                var currentRoute = $location.path().substring(1);
                return page === currentRoute || new RegExp(page).test(currentRoute) ? 'active' : '';
            };

            /**
             * @description Create a new dashboard and save it.
             * @param {boolean|undefined} remote Tells if the dashboard is to create in local storage or on server storage.
             */
            $scope.create = function(remote){
                var id = '_' + Date.now();
                var q = storeService.add(id, defaultDashboard, remote);

                $q.all([q, storeService.getAll()]).then(function(values){
                    takeDashboards(values[1]);

                    $location.path((remote ? '/remote_board/' : '/local_board/') + id);
                    $rootScope.$broadcast('navChanged');
                });
            };
            
            storeService.getAll().then(takeDashboards);

            $scope.$on('navChanged', function(){
                resetDashboards();
                storeService.getAll().then(takeDashboards);
            });
            $scope.$on('updateDashboard', function (event, id, data) {
                storeService.add(id, data).then(function () {
                    $scope.$emit('navChanged');
                }, function (error) {
                    displayError(error);
                });
            });
            $scope.$on('removeDashboard', function (event, id) {
                storeService.remove(id)
                    .then(function () {
                        $scope.$emit('navChanged');
                        $location.path('/');
                    }, function (error) {
                        displayError(error);
                    });
            });

            /* ROUTING ERROR */
            $rootScope.$on('$stateChangeError', function (event, toState, toParams, fromState, fromParams, error) {
                console.log(event, toState, toParams, fromState, fromParams, error);
                
            });
            $rootScope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
                if (toState.name === 'logout') {
                    $rootScope.$broadcast('navChanged');
                    $location.path('/');
                }
            });
        }
    ]).controller('dashboardCtrl', [
        "$scope", "$rootScope", "$location", "$stateParams", "storeService", "data", "defaultDashboard",
        function ($scope, $rootScope, $location, $stateParams, storeService, data, defaultDashboard) {
            
            this.name = $stateParams.id || 'default';
            this.model = data || defaultDashboard;

            this.remove = function(id){
                storeService.remove(id)
                    .then(function () {
                        $rootScope.$broadcast('navChanged');
                        $location.path('/');
                    });
            };

            $scope.$on('adfDashboardChanged', function(event, name, model) {
                storeService.add(name, model).then(function () {
                    $rootScope.$broadcast('navChanged');
                });
            });
        }
    ]).controller('loginCtrl', [
        "$scope", "$location", "loginService", "$rootScope",
        function ($scope, $location, loginService, $rootScope) {
            $scope.username = "";
            $scope.password = "";
            $scope.signup = false;

            $scope.loginError =  false;
            $scope.signupError = false;

            $scope.apply = function () {
                if ($scope.signup) {
                    loginService.signup({username: $scope.username, password: $scope.password})
                        .then(function () {
                            $scope.signupError = false;
                            $rootScope.$broadcast('navChanged');
                            $location.path('/');
                        }, function () {
                            $scope.signupError = true;
                        });
                } else {
                    loginService.login({username: $scope.username, password: $scope.password})
                        .then(function () {
                            $scope.loginError = false;
                            $rootScope.$broadcast('navChanged');
                            $location.path('/');
                        }, function () {
                            $scope.loginError = true;
                        });
                }
            };
        }
    ]);